@@ -29,25 +29,24 @@ fn write_file(f: &str, data: &Vec<u8>) -> std::io::Result<()> {
     Ok(())
 }
 
-fn rsa_sign_fn(data: Vec<u8>) -> Option<Vec<u8>> {
-    let der = read_file("tests/data/rsa.pkcs8").ok()?;
-    let key = signature::RsaKeyPair::from_pkcs8(&der).ok()?;
+fn rsa_sign_fn(data: &Vec<u8>, sign_key: &Vec<u8>) -> Option<Vec<u8>> {
+    let key = signature::RsaKeyPair::from_pkcs8(sign_key).ok()?;
     let mut sig = vec![0; key.public_modulus_len()];
 
     let rng = rand::SystemRandom::new();
 
-    key.sign(&signature::RSA_PKCS1_SHA256, &rng, &data, &mut sig)
+    key.sign(&signature::RSA_PKCS1_SHA256, &rng, data, &mut sig)
         .ok()?;
 
     Some(sig)
 }
 
-fn ec_sign_fn(data: Vec<u8>) -> Option<Vec<u8>> {
-    let der = read_file("tests/data/ec.pkcs8").ok()?;
-    let key = signature::EcdsaKeyPair::from_pkcs8(&signature::ECDSA_P256_SHA256_ASN1_SIGNING, &der)
-        .ok()?;
+fn ec_sign_fn(data: &Vec<u8>, sign_key: &Vec<u8>) -> Option<Vec<u8>> {
+    let key =
+        signature::EcdsaKeyPair::from_pkcs8(&signature::ECDSA_P256_SHA256_ASN1_SIGNING, sign_key)
+            .ok()?;
     let rng = rand::SystemRandom::new();
-    let sig = key.sign(&rng, &data).ok()?;
+    let sig = key.sign(&rng, data).ok()?;
 
     Some(sig.as_ref().to_vec())
 }
@@ -66,8 +65,8 @@ fn get_key_from_pub_key(pub_key: &Vec<u8>) -> Option<Vec<u8>> {
     Some(key.to_vec())
 }
 
-fn rsa_verify_fn(pub_key: Vec<u8>, data: &Vec<u8>, sign: &Vec<u8>) -> Option<bool> {
-    let k = get_key_from_pub_key(&pub_key)?;
+fn rsa_verify_fn(pub_key: &Vec<u8>, data: &Vec<u8>, sign: &Vec<u8>) -> Option<bool> {
+    let k = get_key_from_pub_key(pub_key)?;
     let key = signature::UnparsedPublicKey::new(&signature::RSA_PKCS1_2048_8192_SHA256, &k);
     match key.verify(data, sign) {
         Ok(_) => Some(true),
@@ -75,8 +74,8 @@ fn rsa_verify_fn(pub_key: Vec<u8>, data: &Vec<u8>, sign: &Vec<u8>) -> Option<boo
     }
 }
 
-fn ec_verify_fn(pub_key: Vec<u8>, data: &Vec<u8>, sign: &Vec<u8>) -> Option<bool> {
-    let k = get_key_from_pub_key(&pub_key)?;
+fn ec_verify_fn(pub_key: &Vec<u8>, data: &Vec<u8>, sign: &Vec<u8>) -> Option<bool> {
+    let k = get_key_from_pub_key(pub_key)?;
     let key = signature::UnparsedPublicKey::new(&signature::ECDSA_P256_SHA256_ASN1, &k);
     match key.verify(data, sign) {
         Ok(_) => Some(true),
@@ -136,7 +135,8 @@ fn x509_rsa_root_test() {
         .sign_oid(vec![1, 2, 840, 113549, 1, 1, 11]) /* sha256WithRSAEncryption (PKCS #1) */
         .build();
 
-    let cert = match x.sign(|c| rsa_sign_fn(c)) {
+    let sign_key = read_file("tests/data/rsa.pkcs8").unwrap_or_else(|_| panic!("File not found"));
+    let cert = match x.sign(rsa_sign_fn, &sign_key) {
         Some(c) => c,
         None => panic!("sign() failed"),
     };
@@ -171,7 +171,8 @@ fn x509_rsa_pub_key_test() {
         .sign_oid(vec![1, 2, 840, 113549, 1, 1, 11]) /* sha256WithRSAEncryption (PKCS #1) */
         .build();
 
-    let cert = match x.sign(|c| rsa_sign_fn(c)) {
+    let sign_key = read_file("tests/data/rsa.pkcs8").unwrap_or_else(|_| panic!("File not found"));
+    let cert = match x.sign(rsa_sign_fn, &sign_key) {
         Some(c) => c,
         None => panic!("sign() failed"),
     };
@@ -227,7 +228,8 @@ fn x509_ec_root_test() {
         .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
-    let cert = match x.sign(|c| ec_sign_fn(c)) {
+    let sign_key = read_file("tests/data/ec.pkcs8").unwrap_or_else(|_| panic!("File not found"));
+    let cert = match x.sign(ec_sign_fn, &sign_key) {
         Some(c) => c,
         None => panic!("sign() failed"),
     };
@@ -271,7 +273,8 @@ fn x509_extension_raw() {
         .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
-    let cert = match x.sign(|c| ec_sign_fn(c)) {
+    let sign_key = read_file("tests/data/ec.pkcs8").unwrap_or_else(|_| panic!("File not found"));
+    let cert = match x.sign(ec_sign_fn, &sign_key) {
         Some(c) => c,
         None => panic!("sign() failed"),
     };
@@ -319,7 +322,8 @@ fn x509_key_usage_extension() {
         .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
-    let cert = match x.sign(|c| ec_sign_fn(c)) {
+    let sign_key = read_file("tests/data/ec.pkcs8").unwrap_or_else(|_| panic!("File not found"));
+    let cert = match x.sign(ec_sign_fn, &sign_key) {
         Some(c) => c,
         None => panic!("build() failed"),
     };
@@ -418,8 +422,8 @@ fn x509_rsa_verify() {
         .pub_key()
         .unwrap_or_else(|| panic!("Failed to get Public Key"));
 
-    assert_eq!(x.verify(rsa_verify_fn, pub_key), Some(true));
-    assert_eq!(x.verify(rsa_verify_fn, pub_key2), Some(true));
+    assert_eq!(x.verify(rsa_verify_fn, &pub_key), Some(true));
+    assert_eq!(x.verify(rsa_verify_fn, &pub_key2), Some(true));
 }
 
 #[test]
@@ -433,5 +437,28 @@ fn x509_ec_verify() {
         .pub_key()
         .unwrap_or_else(|| panic!("Failed to get Public Key"));
 
-    assert_eq!(x.verify(ec_verify_fn, pub_key), Some(true));
+    assert_eq!(x.verify(ec_verify_fn, &pub_key), Some(true));
+}
+
+#[test]
+fn x509_key_usage_decoding() {
+    let der =
+        read_file("tests/data/cert_key_usage.der").unwrap_or_else(|_| panic!("File not found"));
+    let x = der
+        .x509_dec()
+        .unwrap_or_else(|| panic!("Failed to deserialize"));
+
+    let key_usage = x
+        .ext
+        .key_usage()
+        .unwrap_or_else(|| panic!("KeyUsage extension is not found"));
+
+    assert_eq!(
+        key_usage,
+        vec![
+            X509KeyUsage::DigitalSignature,
+            X509KeyUsage::KeyEncipherment,
+            X509KeyUsage::DataEncipherment,
+        ]
+    );
 }
